/**************************************************************************
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*     
*     (c) Copyright 2008 Xilinx, Inc.
*     All rights reserved.
*
**************************************************************************/



/**************************************************************************
* Filename:     prodcon.c
* 
* Description:    
* Demonstrate interprocessor communication via a hw mailbox. 
* A producer processor sends data packets to a consumer processor via
* the XPS Mailbox core.
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -------------------------------------------------------
* 1.00a va   10/02/08 First release
*
******************************************************************************/


/***************************** Include Files *********************************/


#include <string.h>
#include <xstatus.h>
#include <xparameters.h>
#include <xmbox.h>
#ifdef __MICROBLAZE__
#include <xtmrctr_l.h>
#include <mb_interface.h>
#else
#include <xcache_l.h>
#include <xtime_l.h>
#endif
#include <stdio.h>

/************************** Constant Definitions *****************************/

#define MSGSIZ  4096

#if XPAR_CPU_ID == 0
const char *role = "CPU0";
#ifdef __MICROBLAZE__
#define SYSTEM_TIMER_BASEADDR   XPAR_XPS_TIMER_0_BASEADDR
#endif
#else
const char *role = "CPU1";
#define SYSTEM_TIMER_BASEADDR   XPAR_XPS_TIMER_1_BASEADDR
#endif

extern char data[];

void    init_hw ();
void    deinit_hw ();
int     init_timer ();
u32     get_timer ();
void    set_timer (u32 time);
float   conv_cycles_to_secs (u32 cycles);

u32 *sendmsg = (u32*) data;
u32  recvmsg[MSGSIZ/sizeof(u32)];
char *hello  = "HELO";

/**************************** Type Definitions *******************************/


/***************** Macros (Inline Functions) Definitions *********************/


/************************** Function Prototypes ******************************/

int prodcon ()
{
    XMbox mbox;
    XMbox_Config *cfg;
    int status, i;
    u32 rcvd, sent, nbytes, ts;

    init_hw ();

    printf ("--------------------------------------------------------------------\r\n");
    printf ("Dual Processor Producer Consumer Demo!\r\n", role);
    printf ("--------------------------------------------------------------------\r\n");
    printf ("Desc -- CPU0 sends data packets to CPU1 via an XPS Mailbox.\r\n");
    printf ("--------------------------------------------------------------------\r\n");

    printf ("(%s)\t: Starts.\r\n", role);
    printf ("(%s)\t: Initializing the mailbox...", role);

#if XPAR_CPU_ID == 0
    cfg = XMbox_LookupConfig (XPAR_XPS_MAILBOX_0_IF_0_DEVICE_ID);
#else
    cfg = XMbox_LookupConfig (XPAR_XPS_MAILBOX_0_IF_1_DEVICE_ID);
#endif

    status = XMbox_CfgInitialize (&mbox, cfg, cfg->BaseAddress);
    if (status != XST_SUCCESS) {
        printf ("failed!\r\n");
        return status;
    }

    printf ("done!\r\n");

    /* First both processors rendezvous with each other. 
       They do this by exchanging HELO messages */

#if XPAR_CPU_ID == 0
    /* Send the hello */
    printf ("(%s)\t: Sending a HELO.\r\n", role);
    XMbox_WriteBlocking (&mbox, (u32*)hello, 4);
    printf ("(%s)\t: Awaiting a HELO...", role);
    while (1) {
        XMbox_ReadBlocking (&mbox, recvmsg, 4);
        if (memcmp (hello, recvmsg, 4) == 0) 
            break;
    }
    printf ("Got it!\r\n");
#else
    printf ("(%s)\t: Awaiting a HELO...", role);
    while (1) {
        /* Try to recv a hello */
        XMbox_ReadBlocking (&mbox, recvmsg, 4);
        if (memcmp (hello, recvmsg, 4) == 0) 
            break;
    }
    printf ("Got it!\r\n");
    
    /* Send back a hello */
    printf ("(%s)\t: Sending back a HELO.\r\n", role);
    XMbox_WriteBlocking (&mbox, (u32*)hello, 4);
#endif                                                             

    /* Now exchange the data */
    for (i = 0; i < 5; i++) {

#if XPAR_CPU_ID == 0
        printf ("\r\n");
        printf ("(%s)\t: Sending data packet (%d) -->\r\n", role, i);
        set_timer (0);
        nbytes = 0;
        while (nbytes != MSGSIZ) {
            /* Write a message to the mbox */
            status = XMbox_Write (&mbox, sendmsg + (nbytes/sizeof(u32)), 
                                  MSGSIZ - nbytes, &sent);
            if (status != XST_FIFO_NO_ROOM)        
                nbytes += sent;
        }              
        ts = get_timer ();
        printf ("(%s)\t: Successfully sent %d bytes in %f S.\r\n", 
                role, MSGSIZ, conv_cycles_to_secs (ts));
#else
        printf ("\r\n");
        printf ("(%s)\t: Receiving data packet (%d) -->\r\n", role, i);
        set_timer (0);
        nbytes = 0;
        while (nbytes != MSGSIZ) {
            /* Read a message from the mbox */
            status = XMbox_Read (&mbox, recvmsg + (nbytes/sizeof(u32)), 
                                 MSGSIZ - nbytes, &rcvd);
            if (status != XST_NO_DATA)
                nbytes += rcvd;
        }
        ts = get_timer ();
        printf ("(%s)\t: Successfully rcvd %d bytes in %f S.\r\n", 
                role, MSGSIZ, conv_cycles_to_secs (ts));
        
        if (memcmp (recvmsg, data, MSGSIZ)) {
            printf ("(%s)\t: Error! Rcvd data does not match expected.\r\n", 
                    role);
            return XST_FAILURE;
        }
        else
            printf ("(%s)\t: Success! Rcvd data does match expected.\r\n", 
                    role);        
#endif                                                             
    }

    printf ("(%s)\t: End of Demo.\r\n", role);
    printf ("--------------------------------------------------------------------\r\n");
    return XST_SUCCESS;
}

int main ()
{
    return prodcon ();
}

void init_hw ()
{
#ifdef __MICROBLAZE__ 
    microblaze_init_icache_range (0, XPAR_MICROBLAZE_CACHE_BYTE_SIZE);
    microblaze_enable_icache ();
    microblaze_init_dcache_range (0, XPAR_MICROBLAZE_DCACHE_BYTE_SIZE);
    microblaze_enable_dcache ();
#else
    XCache_EnableICache (0x00080000);
    XCache_EnableDCache (0x00080000);    
#endif

    init_timer ();
}

void deinit_hw ()
{
#ifdef __MICROBLAZE__ 
    microblaze_disable_icache ();
    microblaze_disable_dcache ();
#else
    XCache_DisableICache ();
    XCache_DisableDCache ();    
#endif
}


int init_timer ()
{

#ifdef __MICROBLAZE__
    /* Set load value = 0, Load timer, clear interrupt bit, start the timer */
    XTmrCtr_mSetLoadReg(SYSTEM_TIMER_BASEADDR, 0, 0);
    XTmrCtr_mSetControlStatusReg(SYSTEM_TIMER_BASEADDR, 0, 
                                 XTC_CSR_INT_OCCURED_MASK 
                                 | XTC_CSR_LOAD_MASK);
    XTmrCtr_mSetControlStatusReg(SYSTEM_TIMER_BASEADDR, 0,
                                 XTC_CSR_ENABLE_TMR_MASK 
                                 | XTC_CSR_AUTO_RELOAD_MASK);
#endif
}

u32 get_timer () 
{
#ifdef __MICROBLAZE__
    return XTmrCtr_mGetTimerCounterReg(SYSTEM_TIMER_BASEADDR, 0);
#else
    XTime time;
    XTime_GetTime (&time);
    return (u32) time;
#endif
}

void set_timer (u32 time)
{
#ifdef __MICROBLAZE__
    /* Set load value = 0, Load timer, clear interrupt bit, start the timer */
    XTmrCtr_mSetLoadReg(SYSTEM_TIMER_BASEADDR, 0, time);
    XTmrCtr_mSetControlStatusReg(SYSTEM_TIMER_BASEADDR, 0, 
                                 XTC_CSR_INT_OCCURED_MASK 
                                 | XTC_CSR_LOAD_MASK);
    XTmrCtr_mSetControlStatusReg(SYSTEM_TIMER_BASEADDR, 0, 
                                 XTC_CSR_ENABLE_TMR_MASK 
                                 | XTC_CSR_AUTO_RELOAD_MASK);
#else
    XTime t;
    t = (XTime)time;
    XTime_SetTime (t);
#endif
}

float conv_cycles_to_secs (u32 cycles)
{
#ifdef __MICROBLAZE__
    return ((float)cycles / XPAR_CPU_CORE_CLOCK_FREQ_HZ);
#else
    return ((float)cycles / XPAR_CPU_PPC405_CORE_CLOCK_FREQ_HZ);
#endif
}
