/**************************************************************************
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*     
*     (c) Copyright 2008 Xilinx, Inc.
*     All rights reserved.
*
**************************************************************************/



/**************************************************************************
* Filename:     <filename>.c
* 
* Description:    
* Demonstrate two processors synchronizing with each other
* to access a single shared console. The synchronization is
* done via xps_mutex locks.
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -------------------------------------------------------
* 1.00a va   10/02/08 First release
*
*
*
******************************************************************************/


/***************************** Include Files *********************************/

#include <xparameters.h>
#include <xmutex.h>
#include <xuartlite_l.h>
#include <xstatus.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef __MICROBLAZE__
#include <xtmrctr_l.h>
#include <mb_interface.h>
#else
#include <xtime_l.h>
#include <xcache_l.h>
#endif


/************************** Constant Definitions *****************************/


#define MUTEX_NUM               0
#define SHAREDCONSOLE_BASEADDR  XPAR_RS232_UART_0_BASEADDR


/**************************** Type Definitions *******************************/


/***************** Macros (Inline Functions) Definitions *********************/



/************************** Function Prototypes ******************************/

void init_lock ();
void init_hw ();
void deinit_hw ();
void MLOCK ();
void MUNLOCK ();
void outbyte (char c);
void udelay (unsigned int usecs);

#ifdef __MICROBLAZE__
#if XPAR_CPU_ID == 0
#define SYSTEM_TIMER_BASEADDR   XPAR_XPS_TIMER_0_BASEADDR
#else
#define SYSTEM_TIMER_BASEADDR   XPAR_XPS_TIMER_1_BASEADDR
#endif
u32 get_timer ();
void set_timer (u32 time);
unsigned int conv_usecs_to_cycles (u32 usecs);
#endif

/* mutex -- Our main mutex lock */

XMutex mutex;

#if XPAR_CPU_ID == 0
volatile unsigned int *sharedstate = (unsigned int *)XPAR_SHARED_BRAM_IF_CNTLR_0_BASEADDR;
#else
volatile unsigned int *sharedstate = (unsigned int *)XPAR_SHARED_BRAM_IF_CNTLR_1_BASEADDR;
#endif

/* We are either CPU1 or CPU2. 
   Code below relies on the value being one of these two */
#if XPAR_CPU_ID == 2
#undef XPAR_CPU_ID
#define XPAR_CPU_ID 1
#endif

unsigned int cpu_status[2] = { 1000, 2000 };

int main ()
{
    int            i;
    unsigned int   oldstate;
    int count = 0;

    init_hw ();
    init_lock();

#if XPAR_CPU_ID == 0
    printf ("--------------------------------------------------------------------\r\n");
    printf ("Dual Processor Synchronization Demo!\r\n");
    printf ("--------------------------------------------------------------------\r\n");
    printf ("Desc -- CPU0 and CPU1 will synchronize their output to this\r\n");
    printf ("        console using XPS Mutex Locks.\r\n");    
    printf ("--------------------------------------------------------------------\r\n");
    printf ("SHAREDCONSOLE: CPU0 and CPU1 rendezvousing...");
    fflush (stdout);
#endif


    /* Rendezvous first to enable co-ordinated output */
#if XPAR_CPU_ID == 0
    *sharedstate = 0x1;
    while (*sharedstate != 0x0)
        ;
#else
    while (*sharedstate != 0x1)
        ;
    printf ("done\r\n");
    *sharedstate = 0x0;
#endif

    while (count++ < 10) {
        
        MLOCK ();

        oldstate = *sharedstate;
        *sharedstate = cpu_status[XPAR_CPU_ID]++;
        printf ("CPU(%d) -- changing shared state from %d", XPAR_CPU_ID, oldstate);
        printf (" to...");
        udelay (500000);
        printf ("%d.\r\n", *sharedstate);
        udelay (500000);

        MUNLOCK ();
        udelay (50000);
    }

    MLOCK ();
    printf ("SHAREDCONSOLE: CPU(%d) Ends.\r\n", XPAR_CPU_ID);
    MUNLOCK ();

#if XPAR_CPU_ID == 0
    while (*sharedstate != 0x0)
        ;
    printf ("SHAREDCONSOLE: End of Demo!\r\n");
    printf ("--------------------------------------------------------------------\r\n");
#else
    *sharedstate = 0x0;
#endif

    deinit_hw ();
}

/* We purposefully redefine outbyte here to force output from printf and xil_printf 
   to go to the UART we want it to go to, rather than the configured STDOUT */

void outbyte (char c)
{
    XUartLite_SendByte(SHAREDCONSOLE_BASEADDR, c);
}

void init_lock ()
{
    XMutex_Config *cfg;
    
    cfg = XMutex_LookupConfig (XPAR_XPS_MUTEX_0_DEVICE_ID);
    if (cfg == NULL) {
        printf ("SHAREDCONSOLE: Startup error!\r\n");
        abort ();
    }
    XMutex_CfgInitialize (&mutex, cfg, cfg->BaseAddress);
}

void MLOCK ()
{
#ifndef NO_SYNC
    XMutex_Lock (&mutex, MUTEX_NUM);                    /* Acquire lock */
#endif
}

void MUNLOCK ()
{
    u32 status;
#ifndef NO_SYNC
    status = XMutex_Unlock (&mutex, MUTEX_NUM);         /* Release lock */
    if (status != XST_SUCCESS)
        abort ();
#endif
}


void init_hw ()
{
#ifdef __MICROBLAZE__ 
    microblaze_init_icache_range (0, XPAR_MICROBLAZE_CACHE_BYTE_SIZE);
    microblaze_enable_icache ();
    microblaze_init_dcache_range (0, XPAR_MICROBLAZE_DCACHE_BYTE_SIZE);
    microblaze_enable_dcache ();

    /* Set load value = 0, Load timer, clear interrupt bit, start the timer */
    XTmrCtr_mSetLoadReg(SYSTEM_TIMER_BASEADDR, 0, 0);
    XTmrCtr_mSetControlStatusReg(SYSTEM_TIMER_BASEADDR, 0, 
                                 XTC_CSR_INT_OCCURED_MASK 
                                 | XTC_CSR_LOAD_MASK);
    XTmrCtr_mSetControlStatusReg(SYSTEM_TIMER_BASEADDR, 0,
                                 XTC_CSR_ENABLE_TMR_MASK 
                                 | XTC_CSR_AUTO_RELOAD_MASK);
#else
    XCache_EnableICache (0x00080000);
    XCache_EnableDCache (0x00080000);    
#endif
}

void deinit_hw ()
{

#ifdef __MICROBLAZE__ 
    microblaze_disable_icache ();
    microblaze_disable_dcache ();
#else
    XCache_DisableICache ();
    XCache_DisableDCache ();    
#endif

}

void udelay (unsigned int us)
{
#ifdef __MICROBLAZE__
    unsigned int cycles =  conv_usecs_to_cycles (us);
    set_timer (0);
    while (get_timer () < cycles)
        ;
#else
    usleep (us);
#endif
}

#ifdef __MICROBLAZE__

u32 get_timer () 
{
    return XTmrCtr_mGetTimerCounterReg(SYSTEM_TIMER_BASEADDR, 0);
}

void set_timer (u32 time)
{
    /* Set load value = 0, Load timer, clear interrupt bit, start the timer */
    XTmrCtr_mSetLoadReg(SYSTEM_TIMER_BASEADDR, 0, time);
    XTmrCtr_mSetControlStatusReg(SYSTEM_TIMER_BASEADDR, 0, 
                                 XTC_CSR_INT_OCCURED_MASK 
                                 | XTC_CSR_LOAD_MASK);
    XTmrCtr_mSetControlStatusReg(SYSTEM_TIMER_BASEADDR, 0, 
                                 XTC_CSR_ENABLE_TMR_MASK 
                                 | XTC_CSR_AUTO_RELOAD_MASK);
}

unsigned int conv_usecs_to_cycles (u32 usecs)
{
    return (unsigned int)(usecs * (XPAR_CPU_CORE_CLOCK_FREQ_HZ/1000000));
}
#endif

