/* $Id: xmbox.c,v 1.6 2007/07/17 17:51:41 meinelte Exp $ */
/******************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2007 Xilinx Inc.
*       All rights reserved.
*
******************************************************************************/
/*****************************************************************************/
/**
*
* @file xmbox.c
*
* Contains required functions for the XMbox driver.
*
* <pre>
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1.00a va            First release
* 1.00a ecm  06/01/07 Cleanup, new coding standard, check into XCS
* </pre>
*
******************************************************************************/

/***************************** Include Files *********************************/

#include "xenv.h"
#include "xbasic_types.h"
#include "xmbox.h"

/************************** Constant Definitions *****************************/


/**************************** Type Definitions *******************************/


/***************** Macros (Inline Functions) Definitions *********************/


/************************** Function Prototypes ******************************/


/************************** Variable Definitions *****************************/

/*****************************************************************************/
/**
*
* Initializes a specific mailbox.
*
*
* @param    InstancePtr is a pointer to the XMbox instance to be worked on.
* @param    CfgPtr is the device configuration structure containing required HW
*           build data.
* @param    EffectiveAddr is the Physical address of the hardware in a Virtual
*           Memory operating system environment. It is the Base Address in a
*           stand alone environment.
**
* @return
*
*   - XST_SUCCESS if initialization was successful
*
* @note
*
* None.
*
******************************************************************************/
int XMbox_CfgInitialize(XMbox * InstancePtr, XMbox_Config *ConfigPtr,
						 u32 EffectiveAddress)
{

	XASSERT_NONVOID(InstancePtr != NULL);

        /* Clear instance memory and make copy of configuration */
        XENV_MEM_FILL(InstancePtr, 0, sizeof(XMbox));
        XENV_MEM_COPY(&InstancePtr->Config, ConfigPtr, sizeof(XMbox_Config));
        
        InstancePtr->Config.BaseAddress = EffectiveAddress;
        InstancePtr->IsReady = XCOMPONENT_IS_READY;
        return XST_SUCCESS;
}

/*****************************************************************************/
/**
*
* Reads requested bytes from the mailbox referenced by InstancePtr,into the
* buffer pointed to by the provided pointer. The number of bytes must be a
* multiple of 4 (bytes). If not, the call will fail in an ASSERT
*
* This function is non blocking.
*
* @param    InstancePtr is a pointer to the XMbox instance to be worked on.
* @param    BufferPtr is the buffer to read the mailbox contents into,
*           aligned to a word boundary.
* @param    RequestedBytes is the number of bytes of data requested.
* @param    BytesRecvdPtr is the memory that is updated with the number of bytes
*           of data actually read.
*
* @return
*
* On success, the number of bytes read is returned through the pointer.  The
* call may return with fewer bytes placed in the buffer than requested  (not
* including zero). This is not necessarily an error condition and indicates
* the amount of data that was currently available in the mailbox.
*
*    - XST_SUCCESS on success.
*    - XST_NO_DATA ifthere was no data in the mailbox.
*
* @note
*
******************************************************************************/
int XMbox_Read(XMbox * InstancePtr, u32 *BufferPtr, u32 RequestedBytes,
			   u32 *BytesRecvdPtr)

{
	u32 NumBytes;

	XASSERT_NONVOID(InstancePtr != NULL);
	XASSERT_NONVOID(!((u32) BufferPtr & 0x3));
	XASSERT_NONVOID(RequestedBytes != 0);
	XASSERT_NONVOID((RequestedBytes %4) == 0);
	XASSERT_NONVOID(BytesRecvdPtr != NULL);

	NumBytes = 0;

	if (InstancePtr->Config.UseFSL == 0){
		/* PLB based access */
		if (XMbox_mIsEmpty(InstancePtr->Config.BaseAddress)) {
				return XST_NO_DATA;
			}

		/*
		 * Read the Mailbox until empty or the length requested is
		 * satisfied
		 */
		do {
			*BufferPtr++ = XMbox_mReadMBox(InstancePtr->Config.BaseAddress);
			NumBytes += 4;
		} while ((NumBytes != RequestedBytes) &&
			 !(XMbox_mIsEmpty(InstancePtr->Config.BaseAddress)));

		*BytesRecvdPtr = NumBytes;
	}
	else {

		/* FSL based Access */

		if (XMbox_mFSLIsEmpty(InstancePtr->Config.SendID)) {
				return XST_NO_DATA;
			}
		/*
		 * Read the Mailbox until empty or the length requested is
		 * satisfied
		 */

		do {
			*BufferPtr++ = XMbox_mFSLReadMBox(InstancePtr->Config.RecvID);
			NumBytes += 4;
		} while ((NumBytes != RequestedBytes) &&
			 !(XMbox_mFSLIsEmpty(InstancePtr->Config.RecvID)));

		*BytesRecvdPtr = NumBytes;
	}

	return XST_SUCCESS;
}

/*****************************************************************************/
/**
*
* Reads requested bytes from the mailbox referenced by InstancePtr,into the
* buffer pointed to by the provided pointer. The number of bytes must be a
* multiple of 4 (bytes). If not, the call will fail in an ASSERT
*
* @param    InstancePtr is a pointer to the XMbox instance to be worked on.
* @param    BufferPtr is the buffer to read the mailbox contents into,
*           aligned to a word boundary.
* @param    RequestedBytes is the number of bytes of data requested.
*
* @return
*
* None, The call blocks until the number of bytes requested are available.
*
* @note
*
*
******************************************************************************/
void XMbox_ReadBlocking(XMbox * InstancePtr, u32 *BufferPtr,
						u32 RequestedBytes)
{
	u32 NumBytes;

	XASSERT_VOID(InstancePtr != NULL);
	XASSERT_VOID(!((u32) BufferPtr & 0x3));
	XASSERT_VOID(RequestedBytes != 0);
	XASSERT_VOID((RequestedBytes %4) == 0);

	NumBytes = 0;

	if (InstancePtr->Config.UseFSL == 0){
		/* PLB based access */
		/* Block while the mailbox FIFO has at-least some data */

		do {
			while (XMbox_mIsEmpty(InstancePtr->Config.BaseAddress));

		/*
		 * Read the Mailbox until empty or the length requested is
		 * satisfied
		 */

			*BufferPtr++ = XMbox_mReadMBox(InstancePtr->Config.BaseAddress);
			NumBytes += 4;
		} while (NumBytes != RequestedBytes);
	}
	else {

		/* FSL based Access */
		/* Block while the mailbox FIFO has at-least some data */

		do {
			while (XMbox_mFSLIsEmpty(InstancePtr->Config.RecvID));

		/*
		 * Read the Mailbox until empty or the length requested is
		 * satisfied
		 */

			*BufferPtr++ = XMbox_mFSLReadMBox(InstancePtr->Config.RecvID);
			NumBytes += 4;
		} while (NumBytes != RequestedBytes);

	}
}

/*****************************************************************************/
/**
* Writes the requested bytes from the buffer pointed to by the provided pointer
* into the mailbox referenced by InstancePtr.The number of bytes must be a
* multiple of 4 (bytes). If not, the call will fail in an ASSERT
*
* This function is non blocking.
*
* @param    InstancePtr is a pointer to the XMbox instance to be worked on.
* @param    BufferPtr is the source data buffer, aligned to a word boundary.
* @param    RequestedBytes is the number of bytes requested to be written
* @param    BytesRecvdPtr points to memory which is updated with the actual
*			number of bytes written, return value.
* @return
*
* On success, the number of bytes successfully written into the destination
* mailbox is returned in the provided pointer. The call may  return with
* zero. This is not necessarily an error condition and indicates that the
* mailbox is currently full.
*
*    - XST_SUCCESS on success.
*    - XST_FIFO_NO_ROOM if the fifo was full.
*
* @note
*
* The provided buffer pointed to by BufferPtr must be aligned to a
* word boundary.
*
******************************************************************************/
int XMbox_Write(XMbox * InstancePtr, u32 *BufferPtr, u32 RequestedBytes,
				 u32 *BytesSentPtr)
{
	u32 NumBytes;

	XASSERT_NONVOID(InstancePtr != NULL);
	XASSERT_NONVOID(!((u32) BufferPtr & 0x3));
	XASSERT_NONVOID(RequestedBytes != 0);
	XASSERT_NONVOID((RequestedBytes %4) == 0);
	XASSERT_NONVOID(BytesSentPtr != NULL);


	NumBytes = 0;

	if (InstancePtr->Config.UseFSL == 0){
			/* PLB based access */

		if (XMbox_mIsFull(InstancePtr->Config.BaseAddress)) {
				return XST_FIFO_NO_ROOM;
			}

		/*
		 * Write to the Mailbox until full or the length requested is
		 * satisfied
		 */

		do {
			XMbox_mWriteMBox(InstancePtr->Config.BaseAddress, *BufferPtr++);
			NumBytes += 4;
		} while ((NumBytes != RequestedBytes) &&
			 !(XMbox_mIsFull(InstancePtr->Config.BaseAddress)));

		*BytesSentPtr = NumBytes;
	}
	else {

		/* FSL based Access */

		if (XMbox_mFSLIsFull(InstancePtr->Config.SendID)){
				return XST_FIFO_NO_ROOM;
			}
		/*
		 * Write to the Mailbox until full or the length requested is
		 * satisfied
		 */

		do {
			XMbox_mFSLWriteMBox(InstancePtr->Config.SendID, *BufferPtr++);
			NumBytes += 4;
		} while ((NumBytes != RequestedBytes) &&
			 !(XMbox_mFSLIsFull(InstancePtr->Config.SendID)));

		*BytesSentPtr = NumBytes;

	}

	return XST_SUCCESS;
}

/*****************************************************************************/
/**
* Writes the requested bytes from the buffer pointed to by the provided pointer
* into the mailbox referenced by InstancePtr.The number of bytes must be a
* multiple of 4 (bytes). If not, the call will fail in an ASSERT
*
* @param    InstancePtr is a pointer to the XMbox instance to be worked on.
* @param    BufferPtr is the source data buffer, aligned to a word boundary.
* @param    RequestedBytes is the number of bytes requested to be written
* @param    Nbytes points to memory which is updated with the actual number of
*           bytes written
* @return
*
* None, The call blocks until the number of bytes requested are written.
*
* @note
*
* The provided buffer pointed to by BufferPtr must be aligned to a
* word boundary.
*
******************************************************************************/
void XMbox_WriteBlocking(XMbox * InstancePtr, u32 *BufferPtr,
						 u32 RequestedBytes)
{
	u32 NumBytes;

	XASSERT_VOID(InstancePtr != NULL);
	XASSERT_VOID(!((u32) BufferPtr & 0x3));
	XASSERT_VOID(RequestedBytes != 0);
	XASSERT_VOID((RequestedBytes %4) == 0);


	NumBytes = 0;

	if (InstancePtr->Config.UseFSL == 0){
		/* PLB based access */
		/* Block while the mailbox FIFO becomes free to transfer at-least one word */
		do {
			while (XMbox_mIsFull(InstancePtr->Config.BaseAddress));

			XMbox_mWriteMBox(InstancePtr->Config.BaseAddress,*BufferPtr++);
			NumBytes += 4;
		} while (NumBytes != RequestedBytes);
	}
	else {

		/* FSL based Access */
		/* Block while the mailbox FIFO becomes free to transfer at-least one word */
		do {
			while (XMbox_mFSLIsFull(InstancePtr->Config.SendID));

			XMbox_mFSLWriteMBox(InstancePtr->Config.SendID,*BufferPtr++);
			NumBytes += 4;
		} while (NumBytes != RequestedBytes);


	}
}
/*****************************************************************************/
/**
*
* Checks to see if there is data available to be read
*
* @param    InstancePtr is a pointer to the XMbox instance to be worked on.
*
* @return 	FALSE if there is data to be read, TRUE is there isn't
*
*
* @note
*
* None.
*
******************************************************************************/
u32 XMbox_IsEmpty(XMbox * InstancePtr)
{
	XASSERT_NONVOID(InstancePtr != NULL);


	if (InstancePtr->Config.UseFSL == 0){
			/* PLB based access */
		return (XMbox_mIsEmpty(InstancePtr->Config.BaseAddress));
	}
	else {

		/* FSL based Access */
		return (XMbox_mFSLIsEmpty(InstancePtr->Config.RecvID));
	}
}

/*****************************************************************************/
/**
*
* Checks to see if there is room in the write FIFO
*
* @param    InstancePtr is a pointer to the XMbox instance to be worked on.
*
* @return 	FALSE if there is room in write FIFO, TRUE is there isn't
*
* @note
*
* None.
*
******************************************************************************/
u32 XMbox_IsFull(XMbox * InstancePtr)
{
	XASSERT_NONVOID(InstancePtr != NULL);

	if (InstancePtr->Config.UseFSL == 0){
			/* PLB based access */
		return (XMbox_mIsFull(InstancePtr->Config.BaseAddress));
	}
	else {

		/* FSL based Access */
		return (XMbox_mFSLIsFull(InstancePtr->Config.SendID));

	}
}
/*****************************************************************************/
/**
*
* Resets the mailbox FIFOs by empting the READ FIFO and making sure the
* Error Status is zero.
*
* @param    InstancePtr is a pointer to the XMbox instance to be worked on.
*
* @return
*
*    - XST_SUCCESS on success.
*	 - XST_FAILURE if there are any outstanding errors.
*
* @note
*
* data from read FIFO is thrown away.
*
******************************************************************************/
int XMbox_Flush(XMbox * InstancePtr)
{
	XASSERT_NONVOID(InstancePtr != NULL);

	if (InstancePtr->Config.UseFSL == 0){
			/* PLB based access */
		do {
			(void) XMbox_mReadMBox(InstancePtr->Config.BaseAddress);
		} while (!(XMbox_mIsEmpty(InstancePtr->Config.BaseAddress)));

	}
	else {

		/* FSL based Access */

		do {
			(void) XMbox_mFSLReadMBox(InstancePtr->Config.RecvID);
		} while (!(XMbox_mFSLIsEmpty(InstancePtr->Config.RecvID)));


	}

	return XST_SUCCESS;
}

