/**************************************************************************
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*     
*     (c) Copyright 2008 Xilinx, Inc.
*     All rights reserved.
*
**************************************************************************/

/**************************************************************************
* Filename:     shm.c
* 
* Description:    
* Demonstrate simple producer consumer interaction via shared memory. 
* Processor 0 is producer, while Processor 1 acts as consumer.
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -------------------------------------------------------
* 1.00a va   10/02/08 First release.
*
*
*
******************************************************************************/


/***************************** Include Files *********************************/


#include <xparameters.h>
#include <stdio.h>
#include <stdlib.h>

void init_hw ();
void deinit_hw ();

/************************** Constant Definitions *****************************/

#ifndef SHM_BASEADDR      
#if XPAR_CPU_ID == 0  
#define SHM_BASEADDR    XPAR_SHARED_BRAM_IF_CNTLR_0_BASEADDR
#else
#define SHM_BASEADDR    XPAR_SHARED_BRAM_IF_CNTLR_1_BASEADDR
#endif
#endif

#define SHM_WRITEAREA()   *((volatile unsigned int*)(SHM_BASEADDR))
#define SHM_READAREA()    *((volatile unsigned int*)(SHM_BASEADDR))
#define SHM_READFLAG()    *((volatile unsigned int*)(SHM_BASEADDR + 0x8))

#define N_PASSES 20

/**************************** Type Definitions *******************************/


/***************** Macros (Inline Functions) Definitions *********************/


/************************** Function Prototypes ******************************/


void delay ()
{
    volatile int i;

    for (i = 0; i < 0xfffff; i++)
        ;
}


int main ()
{
    int i;
    unsigned int data;

    init_hw ();

    printf ("--------------------------------------------------------------------\r\n");
    printf ("Dual Processor Shared Memory Demo!\r\n");
    printf ("--------------------------------------------------------------------\r\n");
    printf ("Desc -- CPU0 produces data into a single shared memory buffer. \r\n");
    printf ("        CPU1 consumes the data. Flags in shared memory are used to \r\n");    
    printf ("        signal production and consumption of data.\r\n");
    printf ("--------------------------------------------------------------------\r\n");
    printf ("SHM: Starts...\r\n");
    
#if XPAR_CPU_ID == 0
    /* CPU0 performs writes to shared memory */
    printf ("SHM: CPU0: I am a writer. Starting...\r\n");
    SHM_READFLAG() = 0;
    
    for (i = 0; i < N_PASSES; i++) {

        /* Write the data and set the flag */
        SHM_WRITEAREA() =  i;
        SHM_READFLAG() = 1;

        printf ("SHM: CPU0: Wrote a 32-bit value to shared memory. Value is: (%d).\r\n", i);
        
        /* Wait for CPU1 to read the value */
        while (SHM_READFLAG() == 1)
            ;

        delay ();
    }
    
    printf ("SHM: CPU0: Done.\r\n");

#else   /* CPU1 */

    printf ("SHM: CPU1: I am a reader. Starting...\r\n");
    while (1) {
        /* Wait for CPU0 to write a value */
        while (SHM_READFLAG() != 1)
            ;

        /* Read the data and reset the flag */
        data = SHM_READAREA ();
        SHM_READFLAG () = 0;

        printf ("SHM: CPU1: Read a 32-bit value from shared memory. Value is: (%d).\r\n", data);
        
        if (data == N_PASSES - 1)
            break;
    }
    
    printf ("SHM: CPU1: Done.\r\n");
#endif

    printf ("SHM: End of Demo.\r\n");
    printf ("--------------------------------------------------------------------\r\n");
    deinit_hw ();
}

void init_hw ()
{
#ifdef __MICROBLAZE__ 
    microblaze_init_icache_range (0, XPAR_MICROBLAZE_CACHE_BYTE_SIZE);
    microblaze_enable_icache ();
    microblaze_init_dcache_range (0, XPAR_MICROBLAZE_DCACHE_BYTE_SIZE);
    microblaze_enable_dcache ();
#else
    XCache_EnableICache (0x00080000);
    XCache_EnableDCache (0x00080000);    
#endif
}

void deinit_hw ()
{

#ifdef __MICROBLAZE__ 
    microblaze_disable_icache ();
    microblaze_disable_dcache ();
#else
    XCache_DisableICache ();
    XCache_DisableDCache ();    
#endif

}

