/* $Id: xmbox_hw.h,v 1.8 2007/08/29 21:56:03 meinelte Exp $ */
/******************************************************************************
*
*	   XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*	   AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*	   SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*	   OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*	   APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*	   THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*	   AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*	   FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*	   WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*	   IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*	   REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*	   INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*	   FOR A PARTICULAR PURPOSE.
*
*	   (c) Copyright 2007 Xilinx Inc.
*	   All rights reserved.
*
******************************************************************************/
/*****************************************************************************/
/**
*
* @file xmbox_hw.h
*
* This header file contains identifiers and low-level driver functions (or
* macros) that can be used to access the device.  The user should refer to the
* hardware device specification for more details of the device operation.
* High-level driver functions are defined in xmbox.h.
*
* <pre>
* MODIFICATION HISTORY:
*
* Ver   Who  Date       Changes
* ----- ---- --------   -----------------------------------------------
* 1.00a va              First release
* 1.00a ecm  06/01/07   Cleanup, new coding standard, check into XCS
* </pre>
*
******************************************************************************/

#ifndef XMBOX_HW_H		/* prevent circular inclusions */
#define XMBOX_HW_H		/* by using protection macros */

#ifdef __cplusplus
extern "C" {
#endif

/***************************** Include Files *********************************/

#include "xbasic_types.h"
#include "xio.h"

#ifdef __MICROBLAZE__
#include "fsl.h"		/* MicroBlaze Specific */
#endif

/************************** Constant Definitions *****************************/

/** @name Register Offset Definitions
 * Register offsets within a mbox, there are multiple
 * mboxes within a single device
 * @{
 */

#define XMB_WRITE_REG_OFFSET    0x00	/**< Mbox write register */
#define XMB_READ_REG_OFFSET     0x08  	/**< Mbox read register */
#define XMB_STATUS_REG_OFFSET   0x10	/**< Mbox status reg  */
#define XMB_ERROR_REG_OFFSET    0x14	/**< Mbox Error reg  */

/*@}*/

/**
 * @name Status register definitions
 * These status bits are used to poll the FIFOs
 * @{
 */

/**
 * Status bits definition.
 */

#define XMB_STATUS_FIFO_EMPTY   0x00000001
#define XMB_STATUS_FIFO_FULL    0x00000002

/* @} */

/**
 * Error bits definition.
 */

#define XMB_ERROR_FIFO_EMPTY   0x00000001
#define XMB_ERROR_FIFO_FULL    0x00000002

/* @} */

/**************************** Type Definitions *******************************/

/***************** Macros (Inline Functions) Definitions *********************/

/* The following macros are for the memory mapped access method */

/*****************************************************************************/
/**
* Read one of the mbox registers.
*
* @param	BaseAddress contains the base address of the mbox device.
* @param	RegOffset contains the offset from the 1st register of the mbox
*		   to select the specific register of the mbox.
*
* @return   The value read from the register, a 32 bit value.
*
* @note	 None.
*
* u32 XMbox_mReadReg(u32 BaseAddress, unsigned RegOffset)
******************************************************************************/
#define XMbox_mReadReg(BaseAddress, RegOffset)		\
	XIo_In32((BaseAddress) + (RegOffset))
/*****************************************************************************/
/**
* Write a specified value to a register of a mbox.
*
* @param	BaseAddress is the base address of the mbox device.
* @param	RegOffset contain the offset from the 1st register of the
*		   mbox to select the specific register of the mbox.
* @param	ValueToWrite is the 32 bit value to be written to the register.
*
* @return   None
*
* void XMbox_mWriteReg(u32 BaseAddress, unsigned RegOffset, u32 ValueToWrite)
******************************************************************************/
#define XMbox_mWriteReg(BaseAddress, RegOffset, ValueToWrite)   \
	XIo_Out32(((BaseAddress) + (RegOffset)), (ValueToWrite))


/*****************************************************************************/
/**
* Write the mbox write register.
*
* @param	BaseAddress contains the base address of the mbox device.
* @param	ValueToWrite contains the value to be written.
*
* @return   None.
*
* @note	 None.
*
* u32 XMbox_mWriteMBox(u32 BaseAddress, u32 ValueToWrite)
******************************************************************************/
#define XMbox_mWriteMBox(BaseAddress, ValueToWrite)  \
	XMbox_mWriteReg (BaseAddress, XMB_WRITE_REG_OFFSET, ValueToWrite)

/*****************************************************************************/
/**
* Read the mbox read FIFO.
*
* @param	BaseAddress contains the base address of the mbox device.
* @return   The value read from the register, a 32 bit value.
*
* @note	 None.
*
* u32 XMbox_mReadMBox(u32 BaseAddress)
******************************************************************************/
#define XMbox_mReadMBox(BaseAddress)				 \
	XMbox_mReadReg (BaseAddress, XMB_READ_REG_OFFSET)

/*****************************************************************************/
/**
* Checks if the READ FIFO is Empty.
*
* @param	BaseAddress contains the base address of the mbox device.
*
* @return   TRUE if empty, FALSE if not empty and has data to be read.
*
* @note	 None.
*
* u32 XMbox_mIsEmpty(u32 BaseAddress)
******************************************************************************/
#define XMbox_mIsEmpty(BaseAddress)				 \
((XMbox_mReadReg (BaseAddress, XMB_STATUS_REG_OFFSET) & XMB_STATUS_FIFO_EMPTY))

/*****************************************************************************/
/**
* Checks if there is room in the Write FIFO.
*
* @param	BaseAddress contains the base address of the mbox device.
*
* @return   FALSE if there is room in FIFO, TRUE if no room for data
*
* @note	 None.
*
* u32 XMbox_mIsFull(u32 BaseAddress)
******************************************************************************/
#define XMbox_mIsFull(BaseAddress)				 \
((XMbox_mReadReg (BaseAddress, XMB_STATUS_REG_OFFSET) & XMB_STATUS_FIFO_FULL))


/*
 * The following macros are for the MicroBlaze specific Fast Simplex Link (FSL)
 * access method
 */


#ifdef __MICROBLAZE__
/*****************************************************************************/
/**
* Read the mbox FIFO using FSL
*
* @param	ID contains the link to the mbox device.
*
* @return   The value read from the FIFO, a 32 bit value.
*
* @note	 None.
*
* u32 XMbox_mFSLReadReg(u32 ID)
******************************************************************************/
#define XMbox_mFSLReadMBox(ID) \
({ \
u32 rVal; \
getdfslx(rVal,(ID),FSL_DEFAULT); \
rVal; \
})

/*****************************************************************************/
/**
* Write a specified value to mbox FIFO using FSL
*
* @param	ID contains the link to the mbox device.
* @param	ValueToWrite is the 32 bit value to be written to the register.
*
* @return   None
*
* void XMbox_mFSLWriteReg(u32 ID, u32 ValueToWrite)
******************************************************************************/
#define XMbox_mFSLWriteMBox(ID, ValueToWrite)  \
putdfslx((ValueToWrite),(ID), FSL_DEFAULT)

/*****************************************************************************/
/**
* Checks if the READ FIFO is Empty.
*
* @param	ID contains the link to the mbox device.
*
* @return   TRUE if empty, FALSE if not empty and has data to be read.
*
* @note	 None.
*
* u32 XMbox_mFSLIsEmpty(u32 ID)
******************************************************************************/
#define XMbox_mFSLIsEmpty(ID)			\
({                                              \
u32 temp, Empty;                                \
tgetdfslx(temp,(ID), FSL_NONBLOCKING);          \
fsl_isinvalid(Empty);                           \
Empty;                                          \
})

/*****************************************************************************/
/**
* Checks if there is room in the Write FIFO.
*
* @param	ID contains the link to the mbox device.
*
* @return   FALSE if there is room in FIFO, TRUE if no room for data
*
* @note	 None.
*
* u32 XMbox_mFSLIsFull(u32 ID)
******************************************************************************/
#define XMbox_mFSLIsFull(ID) \
({                                   \
u32 Full;                            \
tputdfslx((ID), FSL_NONBLOCKING);    \
fsl_isinvalid(Full);                 \
Full;                                \
})

#else

/* these definitions allow the PPC version to compile, empty calls */

#define XMbox_mFSLReadMBox(ID)					0
#define XMbox_mFSLWriteMBox(ID, ValueToWrite)
#define XMbox_mFSLIsEmpty(ID) 					0
#define XMbox_mFSLIsFull(ID)					0

#endif /* __MICROBLAZE__ */



/************************** Function Prototypes ******************************/

/************************** Variable Definitions *****************************/

#ifdef __cplusplus
}
#endif

#endif /* end of protection macro */
