/**************************************************************************
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*     
*     (c) Copyright 2008 Xilinx, Inc.
*     All rights reserved.
*
**************************************************************************/



/**************************************************************************
* Filename:     ipi.c
* 
* Description:    
*
* Demonstrate one processor interrupting another processor via a
* message delivered through a mailbox.
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -------------------------------------------------------
* 1.00a  va  10/02/08 First release
*
*
*
******************************************************************************/


/***************************** Include Files *********************************/

#include <stdio.h>
#include <stdlib.h>
#include <xparameters.h>
#include <xstatus.h>
#include <xmbox.h>
#include <xintc.h>
#ifdef __MICROBLAZE__
#include <mb_interface.h>
#else
#include <xcache_l.h>
#include <xtime_l.h>
#include <xexception_l.h>
#endif

/************************** Constant Definitions *****************************/


/**************************** Type Definitions *******************************/


#if XPAR_CPU_ID == 0
#define SYSTEM_INTC_BASEADDR   XPAR_XPS_INTC_0_BASEADDR
#define MBOX_INTR_ID           XPAR_XPS_INTC_0_XPS_MAILBOX_0_INTERRUPT_0_INTR 
#else
#define SYSTEM_INTC_BASEADDR   XPAR_XPS_INTC_1_BASEADDR
#define MBOX_INTR_ID           XPAR_XPS_INTC_1_XPS_MAILBOX_0_INTERRUPT_1_INTR 
#endif

#define N_INTRS     15

/***************** Macros (Inline Functions) Definitions *********************/


/************************** Function Prototypes ******************************/

void    init_hw ();
void    deinit_hw ();
int     irq_init ();
int     irq_deinit ();
int     irq_enable ();
int     irq_disable ();
int     irq_register_handler (void *func);

XMbox        mbox;
XIntc        sys_intc;
volatile u32 intr_count = 0;
volatile u32 ack_count = 0;


#if XPAR_CPU_ID == 0

int mp_gen_irq ()
{
  u32 sent;

  irq_disable ();
  XMbox_WriteBlocking (&mbox, (u32*)&intr_count, 4); 
  printf ("CPU0: Sent an intr (%d).", intr_count++);     
  fflush (stdout);
  irq_enable ();
}

/* Executed in an ISR */
int mp_handle_ack_irq ()
{
  u32 val;

  XMbox_ReadBlocking (&mbox, &val, 4); 
  printf (" Got an ack: %d.\r\n", val);
  ack_count++;
}

void delay ()
{
  volatile int i;

  for (i = 0; i < 0xfffff; i++)
    ;
}

#else   /* XPAR_CPU_ID == 0 */

/* Executed in an ISR */
int mp_handle_irq ()
{
  u32 val;
  int status;

  /* Remove the IRQ intr from the MBox FIFO */
  XMbox_ReadBlocking (&mbox, &val, 4); 
  printf ("CPU1: Got an intr: %d. Sent an ack: %d.\r\n", 
          val, intr_count++);

  /* Send acknowledge to him */
  XMbox_WriteBlocking (&mbox, &val, 4); 
}
#endif

int main ()
{
    int i;
    XMbox_Config *cfg;
    int prev_ack_count;
    init_hw ();

    printf ("--------------------------------------------------------------------\r\n");
    printf ("Dual Processor Interprocessor Interrupts Demo!\r\n");
    printf ("--------------------------------------------------------------------\r\n");
    printf ("Desc -- CPU0 interrupts CPU1 through an XPS Mailbox.\r\n");
    printf ("        ACK interrupts are sent back by CPU1.\r\n");    
    printf ("--------------------------------------------------------------------\r\n");
    printf ("IPI: Intr generation starts.\r\n");
    
    /* mbox initialization */
#if XPAR_CPU_ID == 0
    cfg = XMbox_LookupConfig (XPAR_XPS_MAILBOX_0_IF_0_DEVICE_ID);
#else
    cfg = XMbox_LookupConfig (XPAR_XPS_MAILBOX_0_IF_1_DEVICE_ID);
#endif

    XMbox_CfgInitialize (&mbox, cfg, cfg->BaseAddress);
    
#if XPAR_CPU_ID == 0   
    irq_register_handler (mp_handle_ack_irq);
    irq_enable ();
    
    while (ack_count != N_INTRS) {
        prev_ack_count = ack_count;
        mp_gen_irq ();
        while (ack_count == prev_ack_count)
            ;
        delay ();
    }
#else  
    irq_register_handler (mp_handle_irq);
    irq_enable ();
    while (intr_count < N_INTRS);
#endif

    printf ("IPI: Intr generation ends.\r\n");
    printf ("End of Demo!\r\n");
    printf ("--------------------------------------------------------------------\r\n");
    deinit_hw ();
}

void init_hw ()
{
#ifdef __MICROBLAZE__ 
    microblaze_init_icache_range (0, XPAR_MICROBLAZE_CACHE_BYTE_SIZE);
    microblaze_enable_icache ();
    microblaze_init_dcache_range (0, XPAR_MICROBLAZE_DCACHE_BYTE_SIZE);
    microblaze_enable_dcache ();
#else
    XCache_EnableICache (0x00080000);
    XCache_EnableDCache (0x00080000);    
#endif

    irq_init ();
}

void deinit_hw ()
{
#ifdef __MICROBLAZE__ 
    microblaze_disable_icache ();
    microblaze_disable_dcache ();
#else
    XCache_DisableICache ();
    XCache_DisableDCache ();    
#endif

    irq_deinit ();
}

int irq_init ()
{
    u32 status;

    status = XIntc_Initialize (&sys_intc, 0);
    if (status != XST_SUCCESS) {
	printf ("ERROR: XIntc_Initialize failed.\r\n");
        exit (1);
    }

    /* IntC ISR should service all interrupts */
    XIntc_SetIntrSvcOption (SYSTEM_INTC_BASEADDR, XIN_SVC_ALL_ISRS_OPTION);

    /* Start the interrupt controller */
    status = XIntc_Start (&sys_intc, XIN_REAL_MODE);
    if (status != XST_SUCCESS) {
	printf ("ERROR: init_int_system: XIntc_Start failed.\r\n");
	exit (1);
    }

#ifdef __MICROBLAZE__
    microblaze_enable_interrupts ();
#else
    XExc_Init ();                               
    XExc_mDisableExceptions (XEXC_NON_CRITICAL);
    XExc_RegisterHandler (XEXC_ID_NON_CRITICAL_INT, (XExceptionHandler)XIntc_DeviceInterruptHandler, (void*)0);
#endif
}

int irq_deinit ()
{
#ifdef __MICROBLAZE__
    microblaze_disable_interrupts ();
#else
    XExc_mDisableExceptions (XEXC_NON_CRITICAL);	 
#endif
}

int irq_enable ()
{
#ifdef __MICROBLAZE__
    microblaze_enable_interrupts ();
#else
    XExc_mEnableExceptions (XEXC_NON_CRITICAL);
#endif
}

int irq_disable ()
{
#ifdef __MICROBLAZE__
    microblaze_disable_interrupts ();
#else
    XExc_mDisableExceptions (XEXC_NON_CRITICAL);
#endif
}

int irq_register_handler (void *func)
{
    XStatus status;
    /* We always assume we are registering and enabling handler for the main
       interrupt we are interested in */
    
    status = XIntc_Connect (&sys_intc, MBOX_INTR_ID, func, NULL);
    if (status != XST_SUCCESS) {
        printf ("ERROR: irq_register_handler: XIntc_Connect failed.\r\n");
        return -1;
    }
    
    /* Enable FSL interrupt */
    XIntc_Enable (&sys_intc, MBOX_INTR_ID);
}

